@@ -18,7 +18,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
-#include <blkid.h>
 #include <getopt.h>
 
 #ifdef HAVE_LIBUUID
@@ -26,61 +25,17 @@
 #endif
 
 #include "c.h"
-#include "closestream.h"
-#include "all-io.h"
-#include "swapheader.h"
-#include "strutils.h"
 #include "nls.h"
+#include "all-io.h"
+#include "strutils.h"
+#include "closestream.h"
+
+#include "swapheader.h"
+#include "swapprober.h"
 
 #define SWAP_UUID_OFFSET	(offsetof(struct swap_header_v1_2, uuid))
 #define SWAP_LABEL_OFFSET	(offsetof(struct swap_header_v1_2, volume_name))
 
-/*
- * Returns new libblkid prober. This function call exit() on error.
- */
-static blkid_probe get_swap_prober(const char *devname)
-{
-	blkid_probe pr;
-	int rc;
-	const char *version = NULL;
-	char *swap_filter[] = { "swap", NULL };
-
-	pr = blkid_new_probe_from_filename(devname);
-	if (!pr) {
-		warn(_("%s: unable to probe device"), devname);
-		return NULL;
-	}
-
-	blkid_probe_enable_superblocks(pr, TRUE);
-	blkid_probe_set_superblocks_flags(pr,
-			BLKID_SUBLKS_LABEL | BLKID_SUBLKS_UUID |
-			BLKID_SUBLKS_VERSION);
-
-	blkid_probe_filter_superblocks_type(pr, BLKID_FLTR_ONLYIN, swap_filter);
-
-	rc = blkid_do_safeprobe(pr);
-	if (rc == -1)
-		warn(_("%s: unable to probe device"), devname);
-	else if (rc == -2)
-		warnx(_("%s: ambivalent probing result, use wipefs(8)"), devname);
-	else if (rc == 1)
-		warnx(_("%s: not a valid swap partition"), devname);
-
-	if (rc == 0) {
-		/* supported is SWAPSPACE2 only */
-		if (blkid_probe_lookup_value(pr, "VERSION", &version, NULL) == 0
-		    && version
-		    && strcmp(version, "1"))
-			warnx(_("%s: unsupported swap version '%s'"),
-						devname, version);
-		else
-			return pr;
-	}
-
-	blkid_free_probe(pr);
-	return NULL;
-}
-
 /* Print the swap partition information */
 static int print_info(blkid_probe pr)
 {
@@ -96,7 +51,12 @@ static int print_info(blkid_probe pr)
 }
 
 /* Change the swap partition info */
+#ifdef HAVE_LIBUUID
 static int change_info(const char *devname, const char *label, const char *uuid)
+#else
+static int change_info(const char *devname, const char *label,
+		       const char *uuid __attribute__((__unused__)))
+#endif
 {
 	int fd;
 
@@ -161,6 +121,10 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(USAGE_HEADER, out);
 	fprintf(out, _(" %s [options] <device>\n"),
 		program_invocation_short_name);
+
+	fputs(USAGE_SEPARATOR, out);
+	fputs(_("Display or change the label or UUID of a swap area.\n"), out);
+
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -L, --label <label> specify a new label\n"
 		" -U, --uuid <uuid>   specify a new uuid\n"), out);
@@ -178,11 +142,11 @@ int main(int argc, char *argv[])
 	int c, rc = -1;
 
 	static const struct option longopts[] = {
-	    { "help",      0, 0, 'h' },
-	    { "version",   0, 0, 'V' },
-	    { "label",     1, 0, 'L' },
-	    { "uuid",      1, 0, 'U' },
-	    { NULL,        0, 0, 0 }
+	    { "help",      no_argument,       NULL, 'h' },
+	    { "version",   no_argument,       NULL, 'V' },
+	    { "label",     required_argument, NULL, 'L' },
+	    { "uuid",      required_argument, NULL, 'U' },
+	    { NULL,        0, NULL, 0 }
 	};
 
 	setlocale(LC_ALL, "");
@@ -209,8 +173,7 @@ int main(int argc, char *argv[])
 #endif
 			break;
 		default:
-			usage(stderr);
-			break;
+			errtryhelp(EXIT_FAILURE);
 		}
 	}
 
