@@ -623,7 +623,7 @@ static char *get_subsystem(char *chain, char *buf, size_t bufsz)
 }
 
 /*
- * Returns complete path to the device, the patch contains all all subsystems
+ * Returns complete path to the device, the patch contains all subsystems
  * used for the device.
  */
 char *sysfs_get_devchain(struct sysfs_cxt *cxt, char *buf, size_t bufsz)
@@ -833,31 +833,35 @@ err:
 }
 
 /*
- * Returns 1 if the device is private LVM device.
+ * Returns 1 if the device is private LVM device. The @uuid (if not NULL)
+ * returns DM device UUID, use free() to deallocate.
  */
-int sysfs_devno_is_lvm_private(dev_t devno)
+int sysfs_devno_is_lvm_private(dev_t devno, char **uuid)
 {
 	struct sysfs_cxt cxt = UL_SYSFSCXT_EMPTY;
-	char *uuid = NULL;
+	char *id = NULL;
 	int rc = 0;
 
 	if (sysfs_init(&cxt, devno, NULL) != 0)
 		return 0;
 
-	uuid = sysfs_strdup(&cxt, "dm/uuid");
+	id = sysfs_strdup(&cxt, "dm/uuid");
 
 	/* Private LVM devices use "LVM-<uuid>-<name>" uuid format (important
 	 * is the "LVM" prefix and "-<name>" postfix).
 	 */
-	if (uuid && strncmp(uuid, "LVM-", 4) == 0) {
-		char *p = strrchr(uuid + 4, '-');
+	if (id && strncmp(id, "LVM-", 4) == 0) {
+		char *p = strrchr(id + 4, '-');
 
 		if (p && *(p + 1))
 			rc = 1;
 	}
 
 	sysfs_deinit(&cxt);
-	free(uuid);
+	if (uuid)
+		*uuid = id;
+	else
+		free(id);
 	return rc;
 }
 
@@ -1097,4 +1101,4 @@ int main(int argc, char *argv[])
 	sysfs_deinit(&cxt);
 	return EXIT_SUCCESS;
 }
-#endif
+#endif /* TEST_PROGRAM_SYSFS */
