@@ -30,8 +30,8 @@
 extern "C" {
 #endif
 
-#define BLKID_VERSION   "2.30.1"
-#define BLKID_DATE      "20-Jul-2017"
+#define BLKID_VERSION   "2.30.2"
+#define BLKID_DATE      "21-Sep-2017"
 
 /**
  * blkid_dev:
