@@ -1181,7 +1181,7 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 	else if (pa && fdisk_partition_has_size(pa)) {
 		stop = start + pa->size;
 		isrel = pa->size_explicit ? 0 : 1;
-		if (!isrel && stop > start)
+		if ((!isrel || !alignment_required(cxt)) && stop > start)
 			stop -= 1;
 	} else {
 		/* ask user by dialog */
@@ -1227,7 +1227,7 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 		}
 	}
 
-	DBG(LABEL, ul_debug("DOS: raw stop: %ju", (uintmax_t) stop));
+	DBG(LABEL, ul_debug("DOS: raw stop: %ju [limit %ju]", (uintmax_t) stop, (uintmax_t) limit));
 
 	if (stop > limit)
 		stop = limit;
@@ -1237,7 +1237,7 @@ static int add_partition(struct fdisk_context *cxt, size_t n,
 		isrel = 0;
 		if (stop > start)
 			stop -= 1;
-		DBG(LABEL, ul_debug("DOS: don't align end os tiny partition [start=%ju, stop=%ju, grain=%lu]",
+		DBG(LABEL, ul_debug("DOS: don't align end of tiny partition [start=%ju, stop=%ju, grain=%lu]",
 			    (uintmax_t)start,  (uintmax_t)stop, cxt->grain));
 	}
 
@@ -1720,10 +1720,11 @@ static int dos_add_partition(struct fdisk_context *cxt,
 			fdisk_ask_menu_add_item(ask, 'l', _("logical"), _("numbered from 5"));
 
 		rc = fdisk_do_ask(cxt, ask);
+		if (!rc)
+			fdisk_ask_menu_get_result(ask, &c);
+		fdisk_unref_ask(ask);
 		if (rc)
 			return rc;
-		fdisk_ask_menu_get_result(ask, &c);
-		fdisk_unref_ask(ask);
 
 		if (c == 'p') {
 			rc = get_partition_unused_primary(cxt, pa, &res);
