@@ -36,7 +36,8 @@ struct fdisk_script {
 	size_t			nlines;
 	struct fdisk_label	*label;
 
-	unsigned int		json : 1;		/* JSON output */
+	unsigned int		json : 1,		/* JSON output */
+				force_label : 1;	/* label: <name> specified */
 };
 
 
@@ -133,7 +134,7 @@ done:
  * fdisk_ref_script:
  * @dp: script pointer
  *
- * Incremparts reference counter.
+ * Increments reference counter.
  */
 void fdisk_ref_script(struct fdisk_script *dp)
 {
@@ -161,7 +162,7 @@ static void fdisk_reset_script(struct fdisk_script *dp)
  * fdisk_unref_script:
  * @dp: script pointer
  *
- * De-incremparts reference counter, on zero the @dp is automatically
+ * Decrements reference counter, on zero the @dp is automatically
  * deallocated.
  */
 void fdisk_unref_script(struct fdisk_script *dp)
@@ -353,6 +354,23 @@ int fdisk_script_get_nlines(struct fdisk_script *dp)
 	return dp->nlines;
 }
 
+/**
+ * fdisk_script_has_force_label:
+ * @dp: script
+ *
+ * Label has been explicitly specified in the script.
+ *
+ * Since: 2.30
+ *
+ * Returns: true if "label: name" has been parsed.
+ */
+int fdisk_script_has_force_label(struct fdisk_script *dp)
+{
+	assert(dp);
+	return dp->force_label;
+}
+
+
 /**
  * fdisk_script_read_context:
  * @dp: script
@@ -403,24 +421,31 @@ int fdisk_script_read_context(struct fdisk_script *dp, struct fdisk_context *cxt
 		struct fdisk_labelitem item;
 		char buf[64];
 
+		/* first-lba */
 		rc = fdisk_get_disklabel_item(cxt, GPT_LABELITEM_FIRSTLBA, &item);
-		if (rc == 0) {
+		if (!rc) {
 			snprintf(buf, sizeof(buf), "%"PRIu64, item.data.num64);
 			rc = fdisk_script_set_header(dp, "first-lba", buf);
 		}
-		if (rc < 0)
-			goto done;
 
-		rc = fdisk_get_disklabel_item(cxt, GPT_LABELITEM_LASTLBA, &item);
-		if (rc == 0) {
+		/* last-lba */
+		if (!rc)
+			rc = fdisk_get_disklabel_item(cxt, GPT_LABELITEM_LASTLBA, &item);
+		if (!rc) {
 			snprintf(buf, sizeof(buf), "%"PRIu64, item.data.num64);
 			rc = fdisk_script_set_header(dp, "last-lba", buf);
 		}
-		if (rc < 0)
-			goto done;
+
+		/* table-length */
+		if (!rc) {
+			size_t n = fdisk_get_npartitions(cxt);
+			if (n != FDISK_GPT_NPARTITIONS_DEFAULT) {
+				snprintf(buf, sizeof(buf), "%zu", n);
+				rc = fdisk_script_set_header(dp, "table-length", buf);
+			}
+		}
 	}
 
-done:
 	DBG(SCRIPT, ul_debugobj(dp, "read context done [rc=%d]", rc));
 	return rc;
 }
@@ -649,7 +674,7 @@ static int write_file_sfdisk(struct fdisk_script *dp, FILE *f)
  * @dp: script
  * @f: output file
  *
- * Writes script @dp to the ile @f.
+ * Writes script @dp to the file @f.
  *
  * Returns: 0 on success, <0 on error.
  */
@@ -699,14 +724,16 @@ static int parse_line_header(struct fdisk_script *dp, char *s)
 	if (strcmp(name, "label") == 0) {
 		if (dp->cxt && !fdisk_get_label(dp->cxt, value))
 			goto done;			/* unknown label name */
+		dp->force_label = 1;
 	} else if (strcmp(name, "unit") == 0) {
 		if (strcmp(value, "sectors") != 0)
 			goto done;			/* only "sectors" supported */
 	} else if (strcmp(name, "label-id") == 0
 		   || strcmp(name, "device") == 0
 		   || strcmp(name, "first-lba") == 0
-		   || strcmp(name, "last-lba") == 0) {
-		;					/* whatever is posssible */
+		   || strcmp(name, "last-lba") == 0
+		   || strcmp(name, "table-length") == 0) {
+		;					/* whatever is possible */
 	} else
 		goto done;				/* unknown header */
 
@@ -728,7 +755,7 @@ static char *next_token(char **str)
 	     *tk_end = NULL,
 	     *end = NULL,
 	     *p;
-	int open_quote = 0;
+	int open_quote = 0, terminated = 0;
 
 	for (p = *str; p && *p; p++) {
 		if (!tk_begin) {
@@ -750,10 +777,36 @@ static char *next_token(char **str)
 
 	if (!tk_end)
 		return NULL;
-	end = isblank(*tk_end) ? (char *) skip_blank(tk_end) : tk_end;
-	if (*end == ',' || *end == ';')
+
+	end = tk_end;
+
+	/* skip closing quotes */
+	if (*end == '"')
 		end++;
 
+	/* token is terminated by blank (or blank is before "," or ";") */
+	if (isblank(*end)) {
+		end = (char *) skip_blank(end);
+		terminated++;
+	}
+
+	/* token is terminated by "," or ";" */
+	if (*end == ',' || *end == ';') {
+		end++;
+		terminated++;
+
+	/* token is terminated by \0 */
+	} else if (!*end)
+		terminated++;
+
+	if (!terminated) {
+		DBG(SCRIPT, ul_debug("unterminated token '%s'", end));
+		return NULL;
+	}
+
+	/* skip extra space after terminator */
+	end = (char *) skip_blank(end);
+
 	*tk_end = '\0';
 	*str = end;
 	return tk_begin;
@@ -877,8 +930,9 @@ static int parse_line_nameval(struct fdisk_script *dp, char *s)
 			}
 
 		} else if (!strncasecmp(p, "bootable", 8)) {
+			/* we use next_token() to skip possible extra space */
 			char *tk = next_token(&p);
-			if (strcmp(tk, "bootable") == 0)
+			if (tk && strcasecmp(tk, "bootable") == 0)
 				pa->boot = 1;
 			else
 				rc = -EINVAL;
@@ -896,11 +950,10 @@ static int parse_line_nameval(struct fdisk_script *dp, char *s)
 			rc = next_string(&p, &pa->name);
 
 		} else if (!strncasecmp(p, "type=", 5) ||
-
-			   !strncasecmp(p, "Id=", 3)) {		/* backward compatiility */
+			   !strncasecmp(p, "Id=", 3)) {		/* backward compatibility */
 			char *type;
 
-			p += (*p == 'I' ? 3 : 5);		/* "Id=" or "type=" */
+			p += ((*p == 'I' || *p == 'i') ? 3 : 5); /* "Id=", "type=" */
 
 			rc = next_string(&p, &type);
 			if (rc)
@@ -1051,7 +1104,7 @@ static int parse_line_valcommas(struct fdisk_script *dp, char *s)
 			if (*p == ',' || *p == ';' || alone_sign(sign, p)) {
 				fdisk_partition_end_follow_default(pa, 1);
 				if (sign == TK_PLUS)
-					/* alone '+' means use all possible space, elone '-' means nothing */
+					/* '+' alone means use all possible space, '-' alone means nothing */
 					pa->resize = FDISK_RESIZE_ENLARGE;
 			} else {
 				int pow = 0;
@@ -1166,7 +1219,7 @@ static int fdisk_script_read_buffer(struct fdisk_script *dp, char *s)
  * @fn_fgets: callback function
  *
  * The library uses fgets() function to read the next line from the script.
- * This default maybe overrided to another function. Note that the function has
+ * This default maybe overridden by another function. Note that the function has
  * to return the line terminated by \n (for example readline(3) removes \n).
  *
  * Return: 0 on success, <0 on error
@@ -1319,13 +1372,15 @@ struct fdisk_script *fdisk_get_script(struct fdisk_context *cxt)
  * @cxt: context
  * @dp: script
  *
- * Associte context @cxt with script @dp and creates a new empty disklabel.
+ * Associate context @cxt with script @dp and creates a new empty disklabel.
  *
  * Returns: 0 on success, <0 on error.
  */
 int fdisk_apply_script_headers(struct fdisk_context *cxt, struct fdisk_script *dp)
 {
 	const char *name;
+	const char *str;
+	int rc;
 
 	assert(cxt);
 	assert(dp);
@@ -1338,7 +1393,20 @@ int fdisk_apply_script_headers(struct fdisk_context *cxt, struct fdisk_script *d
 	if (!name)
 		return -EINVAL;
 
-	return fdisk_create_disklabel(cxt, name);
+	rc = fdisk_create_disklabel(cxt, name);
+	if (rc)
+		return rc;
+
+	str = fdisk_script_get_header(dp, "table-length");
+	if (str) {
+		uintmax_t sz;
+
+		rc = parse_size(str, &sz, NULL);
+		if (rc == 0)
+			rc = fdisk_gpt_set_npartitions(cxt, sz);
+	}
+
+	return rc;
 }
 
 /**
@@ -1498,6 +1566,24 @@ done:
 	return 0;
 }
 
+static int test_tokens(struct fdisk_test *ts, int argc, char *argv[])
+{
+	char *p, *str = argc == 2 ? strdup(argv[1]) : NULL;
+	int i;
+
+	for (i = 1, p = str; p && *p; i++) {
+		char *tk = next_token(&p);
+
+		if (!tk)
+			break;
+
+		printf("#%d: '%s'\n", i, tk);
+	}
+
+	free(str);
+	return 0;
+}
+
 int main(int argc, char *argv[])
 {
 	struct fdisk_test tss[] = {
@@ -1505,6 +1591,7 @@ int main(int argc, char *argv[])
 	{ "--read",    test_read,    "<file>              read PT script from file" },
 	{ "--apply",   test_apply,   "<device> <file>     try apply script from file to device" },
 	{ "--stdin",   test_stdin,   "                    read input like sfdisk" },
+	{ "--tokens",  test_tokens,  "<string>            parse string" },
 	{ NULL }
 	};
 
