@@ -46,7 +46,6 @@
 
 #ifdef HAVE_LIBSELINUX
 # include <selinux/selinux.h>
-# include <selinux/av_permissions.h>
 # include "selinux_utils.h"
 #endif
 
@@ -57,6 +56,11 @@
 # include "auth.h"
 #endif
 
+#ifdef HAVE_LIBREADLINE
+# define _FUNCTION_DEF
+# include <readline/readline.h>
+#endif
+
 struct finfo {
 	char *full_name;
 	char *office;
@@ -134,13 +138,13 @@ static void parse_argv(struct chfn_control *ctl, int argc, char **argv)
 {
 	int index, c, status = 0;
 	static const struct option long_options[] = {
-		{"full-name", required_argument, 0, 'f'},
-		{"office", required_argument, 0, 'o'},
-		{"office-phone", required_argument, 0, 'p'},
-		{"home-phone", required_argument, 0, 'h'},
-		{"help", no_argument, 0, 'u'},
-		{"version", no_argument, 0, 'v'},
-		{NULL, no_argument, 0, '0'},
+		{ "full-name",    required_argument, NULL, 'f' },
+		{ "office",       required_argument, NULL, 'o' },
+		{ "office-phone", required_argument, NULL, 'p' },
+		{ "home-phone",   required_argument, NULL, 'h' },
+		{ "help",         no_argument,       NULL, 'u' },
+		{ "version",      no_argument,       NULL, 'v' },
+		{ NULL, 0, NULL, 0 },
 	};
 
 	while ((c = getopt_long(argc, argv, "f:r:p:h:o:uv", long_options,
@@ -176,7 +180,7 @@ static void parse_argv(struct chfn_control *ctl, int argc, char **argv)
 		case 'u':
 			usage(stdout);
 		default:
-			usage(stderr);
+			errtryhelp(EXIT_FAILURE);
 		}
 		ctl->changed = 1;
 		ctl->interactive = 0;
@@ -222,31 +226,40 @@ static char *ask_new_field(struct chfn_control *ctl, const char *question,
 			   char *def_val)
 {
 	int len;
-	char *ans;
-	char buf[MAX_FIELD_SIZE + 2];
+	char *buf;
+#ifndef HAVE_LIBREADLINE
+	size_t dummy = 0;
+#endif
 
 	if (!def_val)
 		def_val = "";
 	while (true) {
 		printf("%s [%s]: ", question, def_val);
 		__fpurge(stdin);
-		if (fgets(buf, sizeof(buf), stdin) == NULL)
+#ifdef HAVE_LIBREADLINE
+		rl_bind_key('\t', rl_insert);
+		if ((buf = readline(NULL)) == NULL)
+#else
+		if (getline(&buf, &dummy, stdin) < 0)
+#endif
 			errx(EXIT_FAILURE, _("Aborted."));
-		ans = buf;
 		/* remove white spaces from string end */
-		ltrim_whitespace((unsigned char *) ans);
-		len = rtrim_whitespace((unsigned char *) ans);
-		if (len == 0)
+		ltrim_whitespace((unsigned char *) buf);
+		len = rtrim_whitespace((unsigned char *) buf);
+		if (len == 0) {
+			free(buf);
 			return xstrdup(def_val);
-		if (!strcasecmp(ans, "none")) {
+		}
+		if (!strcasecmp(buf, "none")) {
+			free(buf);
 			ctl->changed = 1;
 			return xstrdup("");
 		}
-		if (check_gecos_string(question, ans) >= 0)
+		if (check_gecos_string(question, buf) >= 0)
 			break;
 	}
 	ctl->changed = 1;
-	return xstrdup(ans);
+	return buf;
 }
 
 /*
@@ -373,7 +386,7 @@ static int save_new_data(struct chfn_control *ctl)
 #else /* HAVE_LIBUSER */
 	/* write the new struct passwd to the passwd file. */
 	ctl->pw->pw_gecos = gecos;
-	if (setpwnam(ctl->pw) < 0) {
+	if (setpwnam(ctl->pw, ".chfn") < 0) {
 		warn("setpwnam failed");
 #endif
 		printf(_
@@ -424,7 +437,9 @@ int main(int argc, char **argv)
 #ifdef HAVE_LIBSELINUX
 	if (is_selinux_enabled() > 0) {
 		if (uid == 0) {
-			if (checkAccess(ctl.username, PASSWD__CHFN) != 0) {
+			access_vector_t av = get_access_vector("passwd", "chfn");
+
+			if (selinux_check_passwd_access(av) != 0) {
 				security_context_t user_context;
 				if (getprevcon(&user_context) < 0)
 					user_context = NULL;
