@@ -154,7 +154,7 @@ struct libmnt_cache *cache;
 
 
 #ifdef HAVE_LIBUDEV
-struct udev *udev;
+static struct udev *udev;
 #endif
 
 static int match_func(struct libmnt_fs *fs, void *data __attribute__ ((__unused__)));
@@ -684,12 +684,13 @@ static struct libscols_line *add_line(struct libscols_table *table, struct libmn
 	size_t i;
 	struct libscols_line *line = scols_table_new_line(table, parent);
 
-	if (!line) {
-		warn(_("failed to add line to output"));
-		return NULL;
+	if (!line)
+		err(EXIT_FAILURE, _("failed to allocate output line"));
+
+	for (i = 0; i < ncolumns; i++) {
+		if (scols_line_refer_data(line, i, get_data(fs, i)))
+			err(EXIT_FAILURE, _("failed to add output data"));
 	}
-	for (i = 0; i < ncolumns; i++)
-		scols_line_refer_data(line, i, get_data(fs, i));
 
 	scols_line_set_userdata(line, fs);
 	return line;
@@ -701,13 +702,14 @@ static struct libscols_line *add_tabdiff_line(struct libscols_table *table, stru
 	size_t i;
 	struct libscols_line *line = scols_table_new_line(table, NULL);
 
-	if (!line) {
-		warn(_("failed to add line to output"));
-		return NULL;
+	if (!line)
+		err(EXIT_FAILURE, _("failed to allocate output line"));
+
+	for (i = 0; i < ncolumns; i++) {
+		if (scols_line_refer_data(line, i,
+				get_tabdiff_data(old_fs, new_fs, change, i)))
+			err(EXIT_FAILURE, _("failed to add output data"));
 	}
-	for (i = 0; i < ncolumns; i++)
-		scols_line_refer_data(line, i,
-				get_tabdiff_data(old_fs, new_fs, change, i));
 
 	return line;
 }
@@ -1197,7 +1199,8 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -s, --fstab            search in static table of filesystems\n"), out);
-	fputs(_(" -m, --mtab             search in table of mounted filesystems\n"), out);
+	fputs(_(" -m, --mtab             search in table of mounted filesystems\n"
+		"                          (includes user space mount options)\n"), out);
 	fputs(_(" -k, --kernel           search in kernel table of mounted\n"
 		"                          filesystems (default)\n"), out);
 	fputc('\n', out);
@@ -1229,6 +1232,7 @@ static void __attribute__((__noreturn__)) usage(FILE *out)
 	fputs(_(" -S, --source <string>  the device to mount (by name, maj:min, \n"
 	        "                          LABEL=, UUID=, PARTUUID=, PARTLABEL=)\n"), out);
 	fputs(_(" -T, --target <path>    the path to the filesystem to use\n"), out);
+	fputs(_("     --tree             enable tree format output is possible\n"), out);
 	fputs(_(" -M, --mountpoint <dir> the mountpoint directory\n"), out);
 	fputs(_(" -t, --types <list>     limit the set of filesystems by FS types\n"), out);
 	fputs(_(" -U, --uniq             ignore filesystems with duplicate target\n"), out);
@@ -1264,54 +1268,57 @@ int main(int argc, char *argv[])
 	int ntabfiles = 0, tabtype = 0;
 	char *outarg = NULL;
 	size_t i;
+	int force_tree = 0, istree = 0;
 
 	struct libscols_table *table = NULL;
 
 	enum {
-                FINDMNT_OPT_VERBOSE = CHAR_MAX + 1
+                FINDMNT_OPT_VERBOSE = CHAR_MAX + 1,
+		FINDMNT_OPT_TREE
 	};
 
 	static const struct option longopts[] = {
-	    { "all",          0, 0, 'A' },
-	    { "ascii",        0, 0, 'a' },
-	    { "bytes",        0, 0, 'b' },
-	    { "canonicalize", 0, 0, 'c' },
-	    { "direction",    1, 0, 'd' },
-	    { "df",           0, 0, 'D' },
-	    { "evaluate",     0, 0, 'e' },
-	    { "first-only",   0, 0, 'f' },
-	    { "fstab",        0, 0, 's' },
-	    { "help",         0, 0, 'h' },
-	    { "invert",       0, 0, 'i' },
-	    { "json",         0, 0, 'J' },
-	    { "kernel",       0, 0, 'k' },
-	    { "list",         0, 0, 'l' },
-	    { "mountpoint",   1, 0, 'M' },
-	    { "mtab",         0, 0, 'm' },
-	    { "noheadings",   0, 0, 'n' },
-	    { "notruncate",   0, 0, 'u' },
-	    { "options",      1, 0, 'O' },
-	    { "output",       1, 0, 'o' },
-	    { "poll",         2, 0, 'p' },
-	    { "pairs",        0, 0, 'P' },
-	    { "raw",          0, 0, 'r' },
-	    { "types",        1, 0, 't' },
-	    { "nocanonicalize", 0, 0, 'C' },
-	    { "nofsroot",     0, 0, 'v' },
-	    { "submounts",    0, 0, 'R' },
-	    { "source",       1, 0, 'S' },
-	    { "tab-file",     1, 0, 'F' },
-	    { "task",         1, 0, 'N' },
-	    { "target",       1, 0, 'T' },
-	    { "timeout",      1, 0, 'w' },
-	    { "uniq",         0, 0, 'U' },
-	    { "verify",       0, 0, 'x' },
-	    { "version",      0, 0, 'V' },
-	    { "verbose",      0, 0, FINDMNT_OPT_VERBOSE },
-	    { NULL,           0, 0, 0 }
+		{ "all",	    no_argument,       NULL, 'A'		 },
+		{ "ascii",	    no_argument,       NULL, 'a'		 },
+		{ "bytes",	    no_argument,       NULL, 'b'		 },
+		{ "canonicalize",   no_argument,       NULL, 'c'		 },
+		{ "direction",	    required_argument, NULL, 'd'		 },
+		{ "df",		    no_argument,       NULL, 'D'		 },
+		{ "evaluate",	    no_argument,       NULL, 'e'		 },
+		{ "first-only",	    no_argument,       NULL, 'f'		 },
+		{ "fstab",	    no_argument,       NULL, 's'		 },
+		{ "help",	    no_argument,       NULL, 'h'		 },
+		{ "invert",	    no_argument,       NULL, 'i'		 },
+		{ "json",	    no_argument,       NULL, 'J'		 },
+		{ "kernel",	    no_argument,       NULL, 'k'		 },
+		{ "list",	    no_argument,       NULL, 'l'		 },
+		{ "mountpoint",	    required_argument, NULL, 'M'		 },
+		{ "mtab",	    no_argument,       NULL, 'm'		 },
+		{ "noheadings",	    no_argument,       NULL, 'n'		 },
+		{ "notruncate",	    no_argument,       NULL, 'u'		 },
+		{ "options",	    required_argument, NULL, 'O'		 },
+		{ "output",	    required_argument, NULL, 'o'		 },
+		{ "poll",	    optional_argument, NULL, 'p'		 },
+		{ "pairs",	    no_argument,       NULL, 'P'		 },
+		{ "raw",	    no_argument,       NULL, 'r'		 },
+		{ "types",	    required_argument, NULL, 't'		 },
+		{ "nocanonicalize", no_argument,       NULL, 'C'		 },
+		{ "nofsroot",	    no_argument,       NULL, 'v'		 },
+		{ "submounts",	    no_argument,       NULL, 'R'		 },
+		{ "source",	    required_argument, NULL, 'S'		 },
+		{ "tab-file",	    required_argument, NULL, 'F'		 },
+		{ "task",	    required_argument, NULL, 'N'		 },
+		{ "target",	    required_argument, NULL, 'T'		 },
+		{ "timeout",	    required_argument, NULL, 'w'		 },
+		{ "uniq",	    no_argument,       NULL, 'U'		 },
+		{ "verify",	    no_argument,       NULL, 'x'		 },
+		{ "version",	    no_argument,       NULL, 'V'		 },
+		{ "verbose",	    no_argument,       NULL, FINDMNT_OPT_VERBOSE },
+		{ "tree",	    no_argument,       NULL, FINDMNT_OPT_TREE	 },
+		{ NULL, 0, NULL, 0 }
 	};
 
-	static const ul_excl_t excl[] = {	/* rows and cols in in ASCII order */
+	static const ul_excl_t excl[] = {	/* rows and cols in ASCII order */
 		{ 'C', 'c'},                    /* [no]canonicalize */
 		{ 'C', 'e' },			/* nocanonicalize, evaluate */
 		{ 'J', 'P', 'r','x' },		/* json,pairs,raw,verify */
@@ -1471,9 +1478,11 @@ int main(int argc, char *argv[])
 		case FINDMNT_OPT_VERBOSE:
 			flags |= FL_VERBOSE;
 			break;
-		default:
-			usage(stderr);
+		case FINDMNT_OPT_TREE:
+			force_tree = 1;
 			break;
+		default:
+			errtryhelp(EXIT_FAILURE);
 		}
 	}
 
@@ -1553,7 +1562,11 @@ int main(int argc, char *argv[])
 	if (tabtype == TABTYPE_MTAB && tab_is_kernel(tb))
 		tabtype = TABTYPE_KERNEL;
 
-	if ((flags & FL_TREE) && (ntabfiles > 1 || !tab_is_tree(tb)))
+	istree = tab_is_tree(tb);
+	if (istree && force_tree)
+		flags |= FL_TREE;
+
+	if ((flags & FL_TREE) && (ntabfiles > 1 || !istree))
 		flags &= ~FL_TREE;
 
 	if (!(flags & FL_NOCACHE)) {
@@ -1582,7 +1595,7 @@ int main(int argc, char *argv[])
 	scols_init_debug(0);
 	table = scols_new_table();
 	if (!table) {
-		warn(_("failed to initialize output table"));
+		warn(_("failed to allocate output table"));
 		goto leave;
 	}
 	scols_table_enable_raw(table,        !!(flags & FL_RAW));
@@ -1608,7 +1621,7 @@ int main(int argc, char *argv[])
 		}
 		if (!scols_table_new_column(table, get_column_name(i),
 					get_column_whint(i), fl)) {
-			warn(_("failed to initialize output column"));
+			warn(_("failed to allocate output column"));
 			goto leave;
 		}
 	}
