@@ -25,7 +25,6 @@
  */
 
 #include <errno.h>
-#include <features.h>
 #include <getopt.h>
 #include <sys/time.h>
 #include <unistd.h>
@@ -40,6 +39,7 @@
 #include "xalloc.h"
 #include "procutils.h"
 #include "ipcutils.h"
+#include "timeutils.h"
 
 /*
  * time modes
@@ -249,7 +249,7 @@ static char *get_groupname(struct group **gr, gid_t id)
 	return *gr ? xstrdup((*gr)->gr_name) : NULL;
 }
 
-static int parse_time_mode(const char *optarg)
+static int parse_time_mode(const char *s)
 {
 	struct lsipc_timefmt {
 		const char *name;
@@ -263,10 +263,10 @@ static int parse_time_mode(const char *optarg)
 	size_t i;
 
 	for (i = 0; i < ARRAY_SIZE(timefmts); i++) {
-		if (strcmp(timefmts[i].name, optarg) == 0)
+		if (strcmp(timefmts[i].name, s) == 0)
 			return timefmts[i].val;
 	}
-	errx(EXIT_FAILURE, _("unknown time format: %s"), optarg);
+	errx(EXIT_FAILURE, _("unknown time format: %s"), s);
 }
 
 static void __attribute__ ((__noreturn__)) usage(FILE * out)
@@ -335,7 +335,8 @@ static struct libscols_table *new_table(struct lsipc_control *ctl)
 	struct libscols_table *table = scols_new_table();
 
 	if (!table)
-		errx(EXIT_FAILURE, _("failed to initialize output table"));
+		err(EXIT_FAILURE, _("failed to allocate output table"));
+
 	if (ctl->noheadings)
 		scols_table_enable_noheadings(table, 1);
 
@@ -430,44 +431,27 @@ static int print_table(struct lsipc_control *ctl, struct libscols_table *tb)
 }
 static struct timeval now;
 
-static int date_is_today(time_t t)
-{
-	if (now.tv_sec == 0)
-		gettimeofday(&now, NULL);
-	return t / (3600 * 24) == now.tv_sec / (3600 * 24);
-}
-
-static int date_is_thisyear(time_t t)
-{
-	if (now.tv_sec == 0)
-		gettimeofday(&now, NULL);
-	return t / (3600 * 24 * 365) == now.tv_sec / (3600 * 24 * 365);
-}
-
 static char *make_time(int mode, time_t time)
 {
-	char *s;
-	struct tm tm;
 	char buf[64] = {0};
 
-	localtime_r(&time, &tm);
-
 	switch(mode) {
 	case TIME_FULL:
+	{
+		struct tm tm;
+		char *s;
+
+		localtime_r(&time, &tm);
 		asctime_r(&tm, buf);
 		if (*(s = buf + strlen(buf) - 1) == '\n')
 			*s = '\0';
 		break;
+	}
 	case TIME_SHORT:
-		if (date_is_today(time))
-			strftime(buf, sizeof(buf), "%H:%M", &tm);
-		else if (date_is_thisyear(time))
-			strftime(buf, sizeof(buf), "%b%d", &tm);
-		else
-			strftime(buf, sizeof(buf), "%Y-%b%d", &tm);
+		strtime_short(&time, &now, 0, buf, sizeof(buf));
 		break;
 	case TIME_ISO:
-		strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%S%z", &tm);
+		strtime_iso(&time, ISO_8601_DATE|ISO_8601_TIME|ISO_8601_TIMEZONE, buf, sizeof(buf));
 		break;
 	default:
 		errx(EXIT_FAILURE, _("unsupported time type"));
@@ -483,7 +467,7 @@ static void global_set_data(struct libscols_table *tb, const char *resource,
 
 	ln = scols_table_new_line(tb, NULL);
 	if (!ln)
-		err_oom();
+		err(EXIT_FAILURE, _("failed to allocate output line"));
 
 	for (n = 0; n < ncolumns; n++) {
 		int rc = 0;
@@ -517,7 +501,7 @@ static void global_set_data(struct libscols_table *tb, const char *resource,
 		}
 
 		if (rc != 0)
-			err(EXIT_FAILURE, _("failed to set data"));
+			err(EXIT_FAILURE, _("failed to add output data"));
 	}
 }
 
@@ -555,7 +539,10 @@ static void do_sem(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 	}
 	for (semdsp = semds;  semdsp->next != NULL || id > -1; semdsp = semdsp->next) {
 		size_t n;
+
 		ln = scols_table_new_line(tb, NULL);
+		if (!ln)
+			err(EXIT_FAILURE, _("failed to allocate output line"));
 
 		for (n = 0; n < ncolumns; n++) {
 			int rc = 0;
@@ -579,7 +566,7 @@ static void do_sem(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 					xasprintf(&arg, "%#o", semdsp->sem_perm.mode & 0777);
 				else {
 					arg = xmalloc(11);
-					strmode(semdsp->sem_perm.mode & 0777, arg);
+					xstrmode(semdsp->sem_perm.mode & 0777, arg);
 				}
 				rc = scols_line_refer_data(ln, n, arg);
 				break;
@@ -639,7 +626,7 @@ static void do_sem(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 				break;
 			}
 			if (rc != 0)
-				err(EXIT_FAILURE, _("failed to set data"));
+				err(EXIT_FAILURE, _("failed to add output data"));
 			arg = NULL;
 		}
 
@@ -656,6 +643,9 @@ static void do_sem(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 				struct sem_elem *e = &semds->elements[i];
 				struct libscols_line *sln = scols_table_new_line(sub, NULL);
 
+				if (!sln)
+					err(EXIT_FAILURE, _("failed to allocate output line"));
+
 				/* SEMNUM */
 				xasprintf(&arg, "%zu", i);
 				rc = scols_line_refer_data(sln, 0, arg);
@@ -745,6 +735,9 @@ static void do_msg(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 		size_t n;
 		ln = scols_table_new_line(tb, NULL);
 
+		if (!ln)
+			err(EXIT_FAILURE, _("failed to allocate output line"));
+
 		/* no need to call getpwuid() for the same user */
 		if (!(pw && pw->pw_uid == msgdsp->msg_perm.uid))
 			pw = getpwuid(msgdsp->msg_perm.uid);
@@ -776,7 +769,7 @@ static void do_msg(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 					xasprintf(&arg, "%#o", msgdsp->msg_perm.mode & 0777);
 				else {
 					arg = xmalloc(11);
-					strmode(msgdsp->msg_perm.mode & 0777, arg);
+					xstrmode(msgdsp->msg_perm.mode & 0777, arg);
 					rc = scols_line_refer_data(ln, n, arg);
 				}
 				break;
@@ -902,8 +895,9 @@ static void do_shm(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 	for (shmdsp = shmds; shmdsp->next != NULL || id > -1 ; shmdsp = shmdsp->next) {
 		size_t n;
 		ln = scols_table_new_line(tb, NULL);
+
 		if (!ln)
-			err_oom();
+			err(EXIT_FAILURE, _("failed to allocate output line"));
 
 		for (n = 0; n < ncolumns; n++) {
 			int rc = 0;
@@ -928,7 +922,7 @@ static void do_shm(int id, struct lsipc_control *ctl, struct libscols_table *tb)
 					xasprintf(&arg, "%#o", shmdsp->shm_perm.mode & 0777);
 				else {
 					arg = xmalloc(11);
-					strmode(shmdsp->shm_perm.mode & 0777, arg);
+					xstrmode(shmdsp->shm_perm.mode & 0777, arg);
 				}
 				rc = scols_line_refer_data(ln, n, arg);
 				break;
@@ -1072,14 +1066,13 @@ static void do_shm_global(struct libscols_table *tb)
 			++nsegs;
 			sum_segsz += shmdsp->shm_segsz;
 		}
+		ipc_shm_free_info(shmds);
 	}
 
 	global_set_data(tb, "SHMMNI", _("Shared memory segments"), nsegs, lim.shmmni, 1);
 	global_set_data(tb, "SHMALL", _("Shared memory pages"), sum_segsz / getpagesize(), lim.shmall, 1);
 	global_set_data(tb, "SHMMAX", _("Max size of shared memory segment (bytes)"), 0, lim.shmmax, 0);
 	global_set_data(tb, "SHMMIN", _("Min size of shared memory segment (bytes)"), 0, lim.shmmin, 0);
-
-	ipc_shm_free_info(shmds);
 }
 
 int main(int argc, char *argv[])
@@ -1099,27 +1092,26 @@ int main(int argc, char *argv[])
 	};
 
 	static const struct option longopts[] = {
-		{ "bytes",          no_argument,        0, 'b' },
-		{ "creator",        no_argument,	0, 'c' },
-		{ "export",         no_argument,	0, 'e' },
-		{ "global",         no_argument,	0, 'g' },
-		{ "help",           no_argument,	0, 'h' },
-		{ "id",             required_argument,	0, 'i' },
-		{ "json",           no_argument,	0, 'J' },
-		{ "list",           no_argument,        0, 'l' },
-		{ "newline",        no_argument,	0, 'n' },
-		{ "noheadings",     no_argument,	0, OPT_NOHEAD },
-		{ "notruncate",     no_argument,	0, OPT_NOTRUNC },
-		{ "numeric-perms",  no_argument,	0, 'P' },
-		{ "output",         required_argument,	0, 'o' },
-		{ "pid",            no_argument,	0, 'p' },
-		{ "queues",         no_argument,	0, 'q' },
-		{ "raw",            no_argument,	0, 'r' },
-		{ "semaphores",     no_argument,	0, 's' },
-		{ "shmems",         no_argument,	0, 'm' },
-		{ "time",           no_argument,	0, 't' },
-		{ "time-format",    required_argument,	0, OPT_TIME_FMT },
-		{ "version",        no_argument,	0, 'V' },
+		{ "bytes",          no_argument,        NULL, 'b' },
+		{ "creator",        no_argument,	NULL, 'c' },
+		{ "export",         no_argument,	NULL, 'e' },
+		{ "global",         no_argument,	NULL, 'g' },
+		{ "help",           no_argument,	NULL, 'h' },
+		{ "id",             required_argument,	NULL, 'i' },
+		{ "json",           no_argument,	NULL, 'J' },
+		{ "list",           no_argument,        NULL, 'l' },
+		{ "newline",        no_argument,	NULL, 'n' },
+		{ "noheadings",     no_argument,	NULL, OPT_NOHEAD },
+		{ "notruncate",     no_argument,	NULL, OPT_NOTRUNC },
+		{ "numeric-perms",  no_argument,	NULL, 'P' },
+		{ "output",         required_argument,	NULL, 'o' },
+		{ "queues",         no_argument,	NULL, 'q' },
+		{ "raw",            no_argument,	NULL, 'r' },
+		{ "semaphores",     no_argument,	NULL, 's' },
+		{ "shmems",         no_argument,	NULL, 'm' },
+		{ "time",           no_argument,	NULL, 't' },
+		{ "time-format",    required_argument,	NULL, OPT_TIME_FMT },
+		{ "version",        no_argument,	NULL, 'V' },
 		{NULL, 0, NULL, 0}
 	};
 
@@ -1141,7 +1133,7 @@ int main(int argc, char *argv[])
 
 	scols_init_debug(0);
 
-	while ((opt = getopt_long(argc, argv, "bceghi:Jlmno:PqrstuV", longopts, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "bceghi:Jlmno:PqrstV", longopts, NULL)) != -1) {
 
 		err_exclusive_options(opt, longopts, excl, excl_st);
 
