@@ -175,10 +175,10 @@ static int print_caps(FILE *f, capng_type_t which)
 static void dump_one_secbit(int *first, int *bits, int bit, const char *name)
 {
 	if (*bits & bit) {
-		if (!*first)
-			printf(",");
-		else
+		if (*first)
 			*first = 0;
+		else
+			printf(",");
 		fputs(name, stdout);
 		*bits &= ~bit;
 	}
@@ -206,10 +206,10 @@ static void dump_securebits(void)
 	dump_one_secbit(&first, &bits, SECBIT_KEEP_CAPS_LOCKED,
 			"keep_caps_locked");
 	if (bits) {
-		if (!first)
-			printf(",");
-		else
+		if (first)
 			first = 0;
+		else
+			printf(",");
 		printf("0x%x", (unsigned)bits);
 	}
 
@@ -252,7 +252,7 @@ static void dump_label(const char *name)
 
 static void dump_groups(void)
 {
-	int n = getgroups(0, 0);
+	int n = getgroups(0, NULL);
 	gid_t *groups;
 
 	if (n < 0) {
@@ -602,27 +602,27 @@ int main(int argc, char **argv)
 	};
 
 	static const struct option longopts[] = {
-		{"dump", no_argument, 0, 'd'},
-		{"nnp", no_argument, 0, NNP},
-		{"no-new-privs", no_argument, 0, NNP},
-		{"inh-caps", required_argument, 0, INHCAPS},
-		{"list-caps", no_argument, 0, LISTCAPS},
-		{"ruid", required_argument, 0, RUID},
-		{"euid", required_argument, 0, EUID},
-		{"rgid", required_argument, 0, RGID},
-		{"egid", required_argument, 0, EGID},
-		{"reuid", required_argument, 0, REUID},
-		{"regid", required_argument, 0, REGID},
-		{"clear-groups", no_argument, 0, CLEAR_GROUPS},
-		{"keep-groups", no_argument, 0, KEEP_GROUPS},
-		{"groups", required_argument, 0, GROUPS},
-		{"bounding-set", required_argument, 0, CAPBSET},
-		{"securebits", required_argument, 0, SECUREBITS},
-		{"selinux-label", required_argument, 0, SELINUX_LABEL},
-		{"apparmor-profile", required_argument, 0, APPARMOR_PROFILE},
-		{"help", no_argument, 0, 'h'},
-		{"version", no_argument, 0, 'V'},
-		{NULL, 0, 0, 0}
+		{ "dump",             no_argument,       NULL, 'd'              },
+		{ "nnp",              no_argument,       NULL, NNP              },
+		{ "no-new-privs",     no_argument,       NULL, NNP              },
+		{ "inh-caps",         required_argument, NULL, INHCAPS          },
+		{ "list-caps",        no_argument,       NULL, LISTCAPS         },
+		{ "ruid",             required_argument, NULL, RUID             },
+		{ "euid",             required_argument, NULL, EUID             },
+		{ "rgid",             required_argument, NULL, RGID             },
+		{ "egid",             required_argument, NULL, EGID             },
+		{ "reuid",            required_argument, NULL, REUID            },
+		{ "regid",            required_argument, NULL, REGID            },
+		{ "clear-groups",     no_argument,       NULL, CLEAR_GROUPS     },
+		{ "keep-groups",      no_argument,       NULL, KEEP_GROUPS      },
+		{ "groups",           required_argument, NULL, GROUPS           },
+		{ "bounding-set",     required_argument, NULL, CAPBSET          },
+		{ "securebits",       required_argument, NULL, SECUREBITS       },
+		{ "selinux-label",    required_argument, NULL, SELINUX_LABEL    },
+		{ "apparmor-profile", required_argument, NULL, APPARMOR_PROFILE },
+		{ "help",             no_argument,       NULL, 'h'              },
+		{ "version",          no_argument,       NULL, 'V'              },
+		{ NULL, 0, NULL, 0 }
 	};
 
 	static const ul_excl_t excl[] = {
@@ -750,10 +750,8 @@ int main(int argc, char **argv)
 		case 'V':
 			printf(UTIL_LINUX_VERSION);
 			return EXIT_SUCCESS;
-		case '?':
-			usage(stderr);
 		default:
-			errx(EXIT_FAILURE, _("unrecognized option '%c'"), c);
+			errtryhelp(EXIT_FAILURE);
 		}
 	}
 
@@ -781,9 +779,8 @@ int main(int argc, char **argv)
 		errx(EXIT_FAILURE,
 		     _("--[re]gid requires --keep-groups, --clear-groups, or --groups"));
 
-	if (opts.nnp)
-		if (prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0) == -1)
-			err(EXIT_FAILURE, _("disallow granting new privileges failed"));
+	if (opts.nnp && prctl(PR_SET_NO_NEW_PRIVS, 1, 0, 0, 0) == -1)
+		err(EXIT_FAILURE, _("disallow granting new privileges failed"));
 
 	if (opts.selinux_label)
 		do_selinux_label(opts.selinux_label);
@@ -820,9 +817,8 @@ int main(int argc, char **argv)
 			err(SETPRIV_EXIT_PRIVERR, _("setgroups failed"));
 	}
 
-	if (opts.have_securebits)
-		if (prctl(PR_SET_SECUREBITS, opts.securebits, 0, 0, 0) != 0)
-			err(SETPRIV_EXIT_PRIVERR, _("set process securebits failed"));
+	if (opts.have_securebits && prctl(PR_SET_SECUREBITS, opts.securebits, 0, 0, 0) != 0)
+		err(SETPRIV_EXIT_PRIVERR, _("set process securebits failed"));
 
 	if (opts.bounding_set) {
 		do_caps(CAPNG_BOUNDING_SET, opts.bounding_set);
