@@ -64,7 +64,7 @@
 #include "c.h"
 #include "closestream.h"
 
-wchar_t *buf;
+static wchar_t *buf;
 
 static void sig_handler(int signo __attribute__ ((__unused__)))
 {
@@ -106,9 +106,9 @@ int main(int argc, char *argv[])
 	int ch, rval = EXIT_SUCCESS;
 
 	static const struct option longopts[] = {
-		{ "version",    no_argument,       0, 'V' },
-		{ "help",       no_argument,       0, 'h' },
-		{ NULL,         0, 0, 0 }
+		{ "version",    no_argument,       NULL, 'V' },
+		{ "help",       no_argument,       NULL, 'h' },
+		{ NULL,         0, NULL, 0 }
 	};
 
 	setlocale(LC_ALL, "");
@@ -127,7 +127,7 @@ int main(int argc, char *argv[])
 		case 'h':
 			usage(stdout);
 		default:
-			usage(stderr);
+			errtryhelp(EXIT_FAILURE);
 		}
 
 	argc -= optind;
